'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MessageTransform = exports.InvalidMessageTransformEncodingError = exports.default = exports.PAYLOAD_MAX_SIZE = exports.COMMAND_LENGTH = undefined;

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _neoBlockchainCore = require('neo-blockchain-core');

var _stream = require('stream');

var _payload = require('./payload');

var _Command = require('./Command');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const COMMAND_LENGTH = exports.COMMAND_LENGTH = 12;
const PAYLOAD_MAX_SIZE = exports.PAYLOAD_MAX_SIZE = 0x02000000;

const calculateChecksum = buffer => _neoBlockchainCore.common.toUInt32LE(_neoBlockchainCore.crypto.hash256(buffer));

const deserializeMessageHeader = ({
  context,
  reader
}) => {
  if (reader.readUInt32LE() !== context.messageMagic) {
    throw new _neoBlockchainCore.InvalidFormatError();
  }
  const command = (0, _Command.assertCommand)(reader.readFixedString(COMMAND_LENGTH));
  const length = reader.readUInt32LE();
  if (length > PAYLOAD_MAX_SIZE) {
    throw new _neoBlockchainCore.InvalidFormatError();
  }
  const checksum = reader.readUInt32LE();

  return { command, length, checksum };
};

let Message = function () {
  function Message({ magic, value }) {
    (0, _classCallCheck3.default)(this, Message);
    this.serializeWire = (0, _neoBlockchainCore.createSerializeWire)(this.serializeWireBase.bind(this));

    this.magic = magic;
    this.value = value;
  }

  (0, _createClass3.default)(Message, [{
    key: 'serializeWireBase',
    value: function serializeWireBase(writer) {
      const value = this.value;

      writer.writeUInt32LE(this.magic);
      writer.writeFixedString(value.command, COMMAND_LENGTH);

      let payload = Buffer.alloc(0);
      switch (value.command) {
        case 'addr':
          payload = value.payload.serializeWire();
          break;
        case 'block':
          payload = value.payload.serializeWire();
          break;
        case 'consensus':
          payload = value.payload.serializeWire();
          break;
        case 'filteradd':
          payload = value.payload.serializeWire();
          break;
        case 'filterclear':
          break;
        case 'filterload':
          payload = value.payload.serializeWire();
          break;
        case 'getaddr':
          break;
        case 'getblocks':
          payload = value.payload.serializeWire();
          break;
        case 'getdata':
          payload = value.payload.serializeWire();
          break;
        case 'getheaders':
          payload = value.payload.serializeWire();
          break;
        case 'headers':
          payload = value.payload.serializeWire();
          break;
        case 'inv':
          payload = value.payload.serializeWire();
          break;
        case 'mempool':
          break;
        case 'tx':
          payload = value.payload.serializeWire();
          break;
        case 'verack':
          break;
        case 'version':
          payload = value.payload.serializeWire();
          break;
        case 'alert':
          break;
        case 'merkleblock':
          payload = value.payload.serializeWire();
          break;
        case 'notfound':
          break;
        case 'ping':
          break;
        case 'pong':
          break;
        case 'reject':
          break;
        default:
          // eslint-disable-next-line
          value.command;
          throw new _neoBlockchainCore.InvalidFormatError();
      }

      writer.writeUInt32LE(payload.length);
      writer.writeUInt32LE(calculateChecksum(payload));
      writer.writeBytes(payload);
    }
  }], [{
    key: 'deserializeWireBase',
    value: function deserializeWireBase(options) {
      const { reader, context } = options;
      const { command, length, checksum } = deserializeMessageHeader(options);
      const payloadBuffer = reader.readBytes(length);
      if (calculateChecksum(payloadBuffer) !== checksum) {
        throw new _neoBlockchainCore.InvalidFormatError();
      }

      const payloadOptions = {
        context: options.context,
        buffer: payloadBuffer
      };

      let value;
      switch (command) {
        case 'addr':
          value = {
            command: _Command.COMMAND.ADDR,
            payload: _payload.AddrPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'block':
          value = {
            command: _Command.COMMAND.BLOCK,
            payload: _neoBlockchainCore.Block.deserializeWire(payloadOptions)
          };
          break;
        case 'consensus':
          value = {
            command: _Command.COMMAND.CONSENSUS,
            payload: _payload.ConsensusPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'filteradd':
          value = {
            command: _Command.COMMAND.FILTER_ADD,
            payload: _payload.FilterAddPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'filterclear':
          value = { command: _Command.COMMAND.FILTER_CLEAR };
          break;
        case 'filterload':
          value = {
            command: _Command.COMMAND.FILTER_LOAD,
            payload: _payload.FilterLoadPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'getaddr':
          value = { command: _Command.COMMAND.GET_ADDR };
          break;
        case 'getblocks':
          value = {
            command: _Command.COMMAND.GET_BLOCKS,
            payload: _payload.GetBlocksPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'getdata':
          value = {
            command: _Command.COMMAND.GET_DATA,
            payload: _payload.InvPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'getheaders':
          value = {
            command: _Command.COMMAND.GET_HEADERS,
            payload: _payload.GetBlocksPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'headers':
          value = {
            command: _Command.COMMAND.HEADERS,
            payload: _payload.HeadersPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'inv':
          value = {
            command: _Command.COMMAND.INV,
            payload: _payload.InvPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'mempool':
          value = { command: _Command.COMMAND.MEMPOOL };
          break;
        case 'tx':
          value = {
            command: _Command.COMMAND.TX,
            payload: (0, _neoBlockchainCore.deserializeTransactionWire)(payloadOptions)
          };
          break;
        case 'verack':
          value = { command: _Command.COMMAND.VERACK };
          break;
        case 'version':
          value = {
            command: _Command.COMMAND.VERSION,
            payload: _payload.VersionPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'alert':
          value = { command: _Command.COMMAND.ALERT };
          break;
        case 'merkleblock':
          value = {
            command: _Command.COMMAND.MERKLE_BLOCK,
            payload: _payload.MerkleBlockPayload.deserializeWire(payloadOptions)
          };
          break;
        case 'notfound':
          value = { command: _Command.COMMAND.NOT_FOUND };
          break;
        case 'ping':
          value = { command: _Command.COMMAND.PING };
          break;
        case 'pong':
          value = { command: _Command.COMMAND.PONG };
          break;
        case 'reject':
          value = { command: _Command.COMMAND.REJECT };
          break;
        default:
          // eslint-disable-next-line
          command;
          throw new _neoBlockchainCore.InvalidFormatError();
      }

      return new this({ magic: context.messageMagic, value });
    }
  }, {
    key: 'deserializeWire',
    value: function deserializeWire(options) {
      return this.deserializeWireBase({
        context: options.context,
        reader: new _neoBlockchainCore.BinaryReader(options.buffer)
      });
    }
  }]);
  return Message;
}();

exports.default = Message;

let InvalidMessageTransformEncodingError = exports.InvalidMessageTransformEncodingError = function (_Error) {
  (0, _inherits3.default)(InvalidMessageTransformEncodingError, _Error);

  function InvalidMessageTransformEncodingError() {
    (0, _classCallCheck3.default)(this, InvalidMessageTransformEncodingError);
    return (0, _possibleConstructorReturn3.default)(this, (InvalidMessageTransformEncodingError.__proto__ || (0, _getPrototypeOf2.default)(InvalidMessageTransformEncodingError)).call(this, 'Invalid Message Transform Encoding.'));
  }

  return InvalidMessageTransformEncodingError;
}(Error);

const SIZE_OF_MESSAGE_HEADER = _neoBlockchainCore.IOHelper.sizeOfUInt32LE + _neoBlockchainCore.IOHelper.sizeOfFixedString(COMMAND_LENGTH) + _neoBlockchainCore.IOHelper.sizeOfUInt32LE + _neoBlockchainCore.IOHelper.sizeOfUInt32LE;

let MessageTransform = exports.MessageTransform = function (_Transform) {
  (0, _inherits3.default)(MessageTransform, _Transform);

  function MessageTransform(context) {
    (0, _classCallCheck3.default)(this, MessageTransform);

    var _this2 = (0, _possibleConstructorReturn3.default)(this, (MessageTransform.__proto__ || (0, _getPrototypeOf2.default)(MessageTransform)).call(this, { readableObjectMode: true }));

    _this2.context = context;
    _this2.buffer = Buffer.from([]);
    return _this2;
  }

  (0, _createClass3.default)(MessageTransform, [{
    key: '_transform',
    value: function _transform(chunk, encoding, callback) {
      if (typeof chunk === 'string' || encoding !== 'buffer') {
        throw new InvalidMessageTransformEncodingError();
      }

      this.buffer = Buffer.concat([this.buffer, chunk]);
      try {
        const { remainingBuffer, messages } = this._processBuffer(new _neoBlockchainCore.BinaryReader(this.buffer));
        this.buffer = remainingBuffer;
        messages.forEach(message => this.push(message));
        callback(null);
      } catch (error) {
        callback(error);
      }
    }
  }, {
    key: '_processBuffer',
    value: function _processBuffer(reader) {
      if (reader.remaining < SIZE_OF_MESSAGE_HEADER) {
        return { remainingBuffer: reader.remainingBuffer, messages: [] };
      }

      const { length } = deserializeMessageHeader({
        context: this.context,
        reader: reader.clone()
      });

      if (reader.remaining < SIZE_OF_MESSAGE_HEADER + length) {
        return { remainingBuffer: reader.remainingBuffer, messages: [] };
      }

      const message = Message.deserializeWireBase({
        context: this.context,
        reader
      });
      const { remainingBuffer, messages } = this._processBuffer(reader);
      messages.push(message);
      return { remainingBuffer, messages };
    }
  }]);
  return MessageTransform;
}(_stream.Transform);